/* vim: set ai et ts=4 sw=4: */

#ifndef __file_chooser_h__
#define __file_chooser_h__

#define FILE_CHOOSER(obj)			GTK_CHECK_CAST(obj, file_chooser_get_type(), FileChooser)
#define FILE_CHOOSER_CLASS(klass)	GTK_CHECK_CLASS_CAST(klass, file_chooser_get_type(), FileChooserClass)
#define IS_FILE_CHOOSER(obj)		GTK_CHECK_TYPE(obj, file_chooser_get_type())

typedef enum _FileChooserFlags {
    FILE_CHOOSER_SINGLE = 0x0001,
    FILE_CHOOSER_MULTIPLE = 0x0002,
    FILE_CHOOSER_FILE = 0x0004,
    FILE_CHOOSER_DIR = 0x0008,
    FILE_CHOOSER_SINGLE_FILE = FILE_CHOOSER_SINGLE | FILE_CHOOSER_FILE,
    FILE_CHOOSER_MULTIPLE_FILE = FILE_CHOOSER_MULTIPLE | FILE_CHOOSER_FILE,
    FILE_CHOOSER_SINGLE_DIR = FILE_CHOOSER_SINGLE | FILE_CHOOSER_DIR,
    FILE_CHOOSER_MULTIPLE_DIR = FILE_CHOOSER_MULTIPLE | FILE_CHOOSER_DIR,
    FILE_CHOOSER_SHOW_HIDDEN_FILE = 0x0010,
    FILE_CHOOSER_ICON_VIEW = 0x0100,
    FILE_CHOOSER_TREE_VIEW = 0x0200,
    FILE_CHOOSER_LADDER_VIEW = 0x0400,
    FILE_CHOOSER_ALL_VIEW = 0x0f00,
} FileChooserFlags;

typedef struct _FileChooser {
    GtkDialog parent;

    GtkWidget* notebook;            /* file list view container */
    GtkWidget* select_view_buttons;

    GtkWidget* ok_btn;
    GtkWidget* cancel_btn;

    GtkWidget* tbl;

    GtkWidget* location_hb;

    GtkWidget* location_lbl;
    GtkWidget* location_combo;
    GtkWidget* fav_add_btn;
    GtkWidget* fav_remove_btn;

    GtkWidget** views_rb;//FileListView** views_rb;
    GSList* views_rbg;

    GtkWidget* view;

    GtkWidget* filename_lbl;
    GtkWidget* filename_ent;

    GtkWidget* filter_lbl;
    GtkWidget* filter_combo;


    gint flags; //FileChooserFlags
    gchar* title;
    gchar* file;
    gchar** files;
    gchar* cur_dir;
    gchar** filters;
} FileChooser;

typedef struct _FileChooserClass {
    GtkDialogClass parent_class;
} FileChooserClass;

GtkType file_chooser_get_type(void);

GtkWidget* file_chooser_new(void);

void file_chooser_free(FileChooser* fs);

void file_chooser_set_title(FileChooser* fs, gchar* title);
void file_chooser_set_flags(FileChooser* fs, FileChooserFlags flags);
void file_chooser_set_cur_dir(FileChooser* fs, gchar* cur_dir);
void file_chooser_set_cur_filters(FileChooser* fs, gchar** filters);
gchar* file_chooser_get_file(FileChooser* fs);
gchar** file_chooser_get_files(FileChooser* fs);

#endif /*file-chooser.h*/
