/* vim: set ai et ts=4 sw=4: */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <sys/stat.h>
#include <gnome.h>
#include <libgnomevfs/gnome-vfs.h>
#include "file-list-view.h"

static void   file_list_view_class_init(FileListViewClass* klass);
static void   file_list_view_init(FileListView* view);

static gchar* file_list_view_get_current_dir_unimplemented(FileListView* view);
static void   file_list_view_set_current_dir_unimplemented(FileListView* view,
                                                           const gchar* path);

static GtkWidgetClass *parent_class = NULL;

GType file_list_view_get_type() 
{ 
    static GType file_list_view_type = 0; 

    if(!file_list_view_type) { 
        static GTypeInfo file_list_view_info = { 
            sizeof(FileListViewClass), 
            NULL,
            NULL,
            (GClassInitFunc)file_list_view_class_init, 
            NULL,
            NULL,
            sizeof(FileListView), 
            0,
            (GInstanceInitFunc)file_list_view_init
        }; 

        file_list_view_type = g_type_register_static(GTK_TYPE_VBOX,
                                                     "FileListView",
                                                     &file_list_view_info,
                                                     0); 
    } 

    return file_list_view_type; 
}


static void file_list_view_class_init(FileListViewClass* klass)
{ 
    GtkObjectClass* object_class;
    GtkWidgetClass* widget_class;

    object_class = (GtkObjectClass*)klass;
    widget_class = (GtkWidgetClass*)klass;

    parent_class = g_type_class_peek_parent(klass);

    klass->get_current_dir = file_list_view_get_current_dir_unimplemented;
    klass->set_current_dir = file_list_view_set_current_dir_unimplemented;
}

static void file_list_view_init(FileListView* view)
{ 
    view->show_hidden_files = FALSE;
    view->show_dir_only = FALSE;
    view->select_multiple = FALSE;
}

GtkWidget* file_list_view_new(void)
{
    FileListView* view;

    view = g_object_new(TYPE_FILE_LIST_VIEW, NULL);
    return GTK_WIDGET(view);
} 

static gchar*
file_list_view_get_current_dir_unimplemented(FileListView* view)
{
    g_return_val_if_fail(IS_FILE_LIST_VIEW(view), NULL);

    g_warning ("FileListViewClass::get_current_dir not implemented for `%s'",
               gtk_type_name (GTK_OBJECT_TYPE (view)));

    return NULL;
}

void
file_list_view_set_current_dir_unimplemented(FileListView* view,
                                             const gchar *path)
{
    g_return_if_fail(IS_FILE_LIST_VIEW(view));

    g_warning ("FileListViewClass::set_current_dir not implemented for `%s'",
               gtk_type_name (GTK_OBJECT_TYPE (view)));
}

gchar*
file_list_view_get_current_dir(FileListView* view)
{
    g_return_val_if_fail(IS_FILE_LIST_VIEW(view), NULL);

    return FILE_LIST_VIEW_GET_CLASS(view)->get_current_dir(view);
}

void
file_list_view_set_current_dir(FileListView* view, const gchar *path)
{
    g_return_if_fail(IS_FILE_LIST_VIEW(view));

    FILE_LIST_VIEW_GET_CLASS(view)->set_current_dir(view, path);
}


void file_list_view_set_show_hidden_files(FileListView* view, gboolean show_hidden_files)
{
    view->show_hidden_files = show_hidden_files;
}


void file_list_view_set_show_dir_only(FileListView* view, gboolean show_dir_only)
{
    view->show_dir_only = show_dir_only;
}


void file_list_view_set_select_multiple(FileListView* view, gboolean select_multiple)
{
    view->select_multiple = select_multiple;
}

/*file-list-view.c*/
